/**
 * Check the new password against those listed in the history file
 */

#include <unistd.h>
#include <sys/file.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <crypt.h>
#include "/usr/include/packer.h"
#include "pam_password.h"
#include "logging.h"
#include "util.h"
#include "history.h"

int
checkHist(pam_handle_t *pamh, const char *pw)
{
  char histline[RANDOM_LENGTH];
  char curhist[RANDOM_LENGTH];
  char *user;
  char output[RANDOM_LENGTH];
  char *histchar;
  char *curpass;
  char *hash;
  char *salt[3];
  char oldpass[HIST_LENGTH][MD5_LENGTH];
  int userfound=0;
  int loop;
  int retval=0;
  int result;
  FILE *histfile;
  FILE *newhist;

  result = PAM_GET_ITEM(pamh, PAM_USER, (void **)&user);
  if ( result != PAM_SUCCESS )
  {
	pamErrorMessage(pamh, "Couldn't get pam user? Aborting.");
	return 1;
  } 
  strncpy(output,"Changing password for: ", RANDOM_LENGTH);
  strncat(output,user,RANDOM_LENGTH);
  pamErrorMessage(pamh, output);

  if ( histfile = fopen(PASS_HISTORY, "r") )
  {
     if ( ! (newhist = fopen(NEW_PASS_HISTORY, "w+")) )
     {
        pamErrorMessage(pamh, "Could not open temporary histfile");
        fclose(histfile);
        return 1;
     }
     result = chmod(NEW_PASS_HISTORY,S_IRUSR|S_IWUSR);
     if ( result == -1 )
     {
	strncpy(output,"Couldn't set permissions on NEW_PASS_HISTORY - ", RANDOM_LENGTH);
	strncat(output,strerror(errno),RANDOM_LENGTH);
	pamErrorMessage(pamh,output);
	return 1;
     }
     while ( !feof(histfile) ) {
        fgets(curhist, RANDOM_LENGTH, histfile);
        if (strncmp(curhist, user, strlen(user)) == 0 )
        {
           userfound=1;
           strncpy(histline,curhist,RANDOM_LENGTH);
        } else {
           if ( strncmp(output,curhist,strlen(curhist)) != 0)
           {
                strncpy(output,curhist,RANDOM_LENGTH);
                fprintf(newhist,"%s",curhist);
           }
        }
     }
     fclose(histfile);

     loop=0;
     while(output[loop] != '\0' )
     {
       if(output[loop] == '\0')
         break;
       if(output[loop] == '\n')
         output[loop] = '\0';
       loop++;
     }

     if ( userfound )
     {
        hash=crypt(pw, HISTORY_SALT);
	strncpy(output,user, RANDOM_LENGTH);
	strncat(output,":", 1);
	strncat(output,hash,MD5_LENGTH); 
        histchar = strchr(histline,':');
        curpass = strtok(histchar,":");
        for(loop=1;loop!=HIST_LENGTH;loop++)
        {
           strncpy(oldpass[loop],curpass, MD5_LENGTH);
           strncpy(salt,curpass,2);
           hash = crypt(pw, salt);
           if ( strncmp(hash, oldpass[loop], (MD5_LENGTH-1)) == 0 )
           {
	      // this password has been used before.
	      pamErrorMessage(pamh, "This password has already been used.\n");
              retval=1;
           }
	   strncat(output,":", 1);
	   strncat(output,curpass,MD5_LENGTH);
           curpass = strtok(NULL,":");
           if( curpass == NULL )
                break;
        }
     } else {
	hash=crypt(pw, HISTORY_SALT);
        strncpy(output,user,RANDOM_LENGTH);
	strncat(output,":", 1);
	strncat(output,hash,MD5_LENGTH);
     }

     if (retval == 0)
     {
	if ( (strchr(output,'\n')) == NULL )
	{
        	fprintf(newhist,"%s\n",output);
	} else {
        	fprintf(newhist,"%s",output);
	}
        fclose(newhist);
        unlink(PASS_HISTORY);
        rename(NEW_PASS_HISTORY,PASS_HISTORY);
     } else {
        fclose(newhist);
        unlink(NEW_PASS_HISTORY);
     }
  } else {
	// Couldn't open the history file
	pamErrorMessage(pamh, "Could not open Password history file.\n");
	retval=1;
  }

  return retval;
}

